# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

"""Collection of access filters that control lockd authentication."""

from __future__ import unicode_literals, print_function
import abc
import netaddr


class AccessFilter(object):
    """Access policy object that is used to control client access."""
    # pylint: disable=W0613,R0201

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def check(self, request_handler):
        """Inspect `request_handler` and return True if access is allowed."""
        return True

    @abc.abstractmethod
    def decline(self, request_handler):
        """Perform actions on `request_handler` to signal refusal.

        If an AccessFilter returns False in check() for a given
        request_handler, the decline() method will be called afterwards
        to perform all actions on the request_handler that are needed to
        signal the proper request refusal to the client. This involves
        setting the reponse code and possibly adding headers.
        """
        pass


class NetworkAccess(AccessFilter):
    """Grant access depending on the client's address."""
    # pylint: disable=W0231

    def __init__(self, allowed=None):
        self.allowed = map(netaddr.IPNetwork, allowed or [])

    def check(self, request_handler):
        """True if the client connects from one of the allowed networks."""
        if not self.allowed:
            return True
        client = netaddr.IPAddress(request_handler.client_address[0])
        return any((client in net for net in self.allowed))

    def decline(self, request_handler):
        """Signal access forbidden to client."""
        request_handler.send_error(403)


class PasswordAccess(AccessFilter):
    """Grant access depending on HTTP basic auth password.

    The HTTP user name is currently ignored. There ist just a simple PSK
    verification.
    """
    # pylint: disable=W0231

    def __init__(self, secret, realm='gocept.lockd'):
        self.secret = secret
        self.realm = realm

    def check(self, request_handler):
        """True if the supplied password equals the stored secret."""
        try:
            method, creds = request_handler.headers['Authorization'].split()
            _, password = creds.decode('base64').split(':')
        except (KeyError, ValueError):
            return False
        return method == 'Basic' and self.secret == password

    def decline(self, request_handler):
        request_handler.send_response(401)
        request_handler.send_header('WWW-Authenticate',
                                    'Basic realm="%s"' % self.realm)
        request_handler.end_headers()
