# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

"""lockd domain model"""

from __future__ import unicode_literals, print_function
from .exceptions import AlreadyLocked, NotLocked, LockMismatch
import contextlib
import fcntl
import glob
import os
import zc.lockfile


@contextlib.contextmanager
def open_lockfile(lockfile):
    """Generic lock file opening/closing functionality.

    The open and flock'ed lock file is passed to the with block. After
    the block has exited, the lock file is persisted to disk.
    """
    if not os.path.exists(lockfile):
        open(lockfile, 'w').close()
    with open(lockfile, 'r+') as fobj:
        fcntl.flock(fobj, fcntl.LOCK_EX)
        yield fobj
        fobj.flush()
        os.fsync(fobj)


class Lock(object):
    """Represent a single lock file."""

    def __init__(self, lockfile):
        self.lockfile = lockfile

    def acquire(self, client):
        """Get lock for `client` if none exists.

        Active locks contain the name of their client in the lock file.
        """
        with open_lockfile(self.lockfile) as fobj:
            locked_by = fobj.read().strip()
            if locked_by and client != locked_by:
                raise AlreadyLocked(locked_by)
            fobj.seek(0)
            fobj.truncate()
            print(client, file=fobj)

    def release(self, client):
        """Free up lock if previously held by `client`.

        Inactive locks have empty lock files.
        """
        with open_lockfile(self.lockfile) as fobj:
            locked_by = fobj.read().strip()
            if not locked_by:
                raise NotLocked()
            if client != locked_by:
                raise LockMismatch(locked_by)
            fobj.seek(0)
            fobj.truncate()


class Lockd(object):
    """Lock directory storing individual lock files."""

    def __init__(self, lockdir):
        """Create lockd server storing locks in `lockdir`."""
        try:
            os.makedirs(lockdir)
        except OSError:
            pass
        self.lockdir = lockdir
        self.pidfilename = os.path.join(lockdir, 'lockd.pid')
        self.pidfile = zc.lockfile.LockFile(self.pidfilename)

    def resource_lock_name(self, resource):
        """Return file name for `resource`s lock file."""
        return os.path.join(self.lockdir, resource + '.resource.lock')

    def lock(self, resource, client):
        """Lock `resource` by `client`."""
        Lock(self.resource_lock_name(resource)).acquire(client)

    def unlock(self, resource, client):
        """Unlock `resource` if previously held by `client`."""
        Lock(self.resource_lock_name(resource)).release(client)

    def cleanup(self):
        """Remove empty lock files and pid files on server exit."""
        for lockfile in glob.glob(
            os.path.join(self.lockdir, '*.resource.lock')):
            if not os.path.getsize(lockfile):
                os.unlink(lockfile)
        self.pidfile.close()
        os.unlink(self.pidfilename)
