# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

"""lockd server main loop and dependency pull-together"""

from __future__ import unicode_literals, print_function
from .access import NetworkAccess, PasswordAccess
from .addrinfo import AddrInfo
from .configparser import ListConfigParser
from .lockd import Lockd
from .xmlrpc import XMLRPCServer
import select
import signal
import sys


class Servers(object):
    """Main server controller for multiple sockets.

    Creates XMLRPCServer objects for every listening socket and runs the
    poll loop. Please note that requests are served synchronously. This
    means that serving a request on one connection will block all other
    connections from making progress.
    """

    def __init__(self, listen, port):
        """Creates XML-RPC servers for each address.

        `listen` is a list of IP addresses or host names. `port` is a
        port number or service name.
        """
        self.servers = {}
        self.poll_list = select.poll()
        for addr in listen:
            for addrinfo in AddrInfo.resolve_listen_tcp(addr, port):
                server = XMLRPCServer(addrinfo)
                server.register_introspection_functions()
                self.servers[server.fileno()] = server
                self.poll_list.register(server.fileno())

    def register_instance(self, lockd):
        """Register `lockd` domain object for all servers."""
        for server in self.servers.values():
            server.register_instance(lockd)

    def register_filter(self, *filters):
        """Register all `filters` given for all servers."""
        for server in self.servers.values():
            server.access_filters.extend(filters)

    def run(self, interval=1000):
        """Serve requests on all sockets until stopped."""
        while True:
            for fd_event in self.poll_list.poll(interval):
                self.servers[fd_event[0]].handle_request()


def main(configfile):
    """Console script entry point for lockd server."""
    config = ListConfigParser(configfile)
    server = Servers(config.getlist('lockd', 'listen'),
                     config.get('lockd', 'port'))
    server.register_filter(NetworkAccess(
        config.getlist('lockd', 'allow-hosts', default=[])))
    server.register_filter(PasswordAccess(config.get('lockd', 'secret')))
    lockd = Lockd(config.get('lockd', 'lockdir'))
    server.register_instance(lockd)

    def sighandler(_signum, _frame):
        """Clean up on lockdctl termination signal."""
        lockd.cleanup()
        sys.exit(0)

    signal.signal(signal.SIGTERM, sighandler)
    server.run()
