# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

from __future__ import unicode_literals, print_function
from ..access import NetworkAccess, PasswordAccess
import mock
import unittest


class NetworkAccessTest(unittest.TestCase):

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_accept_if_no_allowed_nets_given(self, reqhandler):
        a = NetworkAccess()
        reqhandler.client_address = ('192.0.2.1', 3510)
        self.assertTrue(a.check(reqhandler))
        reqhandler.client_address = ('2001:db8::1', 10953)
        self.assertTrue(a.check(reqhandler))

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_accept_allowed_client(self, reqhandler):
        a = NetworkAccess(['192.0.2.0/24', '2001:db8::/64'])
        reqhandler.client_address = ('192.0.2.250', 3510)
        self.assertTrue(a.check(reqhandler), 'address not accepted')
        reqhandler.client_address = ('2001:db8::5:f0', 10953)
        self.assertTrue(a.check(reqhandler), 'address not accepted')

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_reject_disallowed_client(self, reqhandler):
        a = NetworkAccess(['192.0.2.0/24', '2001:db8::/64'])
        reqhandler.client_address = ('198.51.100.99', 4781)
        self.assertFalse(a.check(reqhandler), 'address not rejected')
        reqhandler.client_address = ('2001:db8:1::1', 8585)
        self.assertFalse(a.check(reqhandler), 'address not rejected')

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_decline(self, reqhandler):
        a = NetworkAccess()
        a.decline(reqhandler)
        reqhandler.send_error.assert_called_with(403)


def http_basic_auth_header(username, password):
    """Construct HTTP basic auth header from `username` and `password`."""
    return dict(Authorization='Basic %s' % (username + ':' + password).encode(
        'base64'))


class PasswordAccessTest(unittest.TestCase):

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_correct_password(self, reqhandler):
        a = PasswordAccess('secret')
        reqhandler.headers = http_basic_auth_header('user', 'secret')
        self.assertTrue(a.check(reqhandler), 'password check not passed')

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_incorrect_password(self, reqhandler):
        a = PasswordAccess('right')
        reqhandler.headers = http_basic_auth_header('user', 'wrong')
        self.assertFalse(a.check(reqhandler), 'password check passed')

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_no_password(self, reqhandler):
        a = PasswordAccess('right')
        reqhandler.headers = {}
        self.assertFalse(a.check(reqhandler), 'password check passed')

    @mock.patch('SimpleXMLRPCServer.SimpleXMLRPCRequestHandler')
    def test_decline(self, reqhandler):
        a = PasswordAccess('secret')
        a.decline(reqhandler)
        reqhandler.send_response.assert_called_with(401)
